<?php
jimport('joomla.plugin.plugin');
/**
 *
 */
class plgSystemShipstation extends JPlugin {

	var $name = 'shipstation';

	/**
	 * Plugin constructor
	 * Initialize the plugin parameters if required.
	 */
	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);

		if(isset($this->params))
			return;

		$plugin = JPluginHelper::getPlugin('system', 'shipstation');
		if(version_compare(JVERSION,'2.5','<')) {
			jimport('joomla.html.parameter');
			$this->params = new JParameter(@$plugin->params);
		} else {
			$this->params = new JRegistry(@$plugin->params);
		}
	}

	/**
	 * Trigger redirection
	 */
	public function afterInitialise() {
		return $this->onAfterInitialise();
	}

	/**
	 * Trigger redirection
	 */
	public function afterRoute() {
		return $this->onAfterRoute();
	}

	/**
	 * Joomla Trigger : On After Initialise
	 */
	public function onAfterInitialise() {
		$app = JFactory::getApplication();
		if($app->isAdmin())
			return;

		// By default we process during the After Initialise.
		// If the option is desactivate, we exit the function so the "After Route" will check it.
		//
		if(!$this->params->get('after_init', 1))
			return;

		$this->process();
	}

	/**
	 * Joomla Trigger : On After Route
	 */
	public function onAfterRoute() {
		$app = JFactory::getApplication();
		if($app->isAdmin())
			return;

		// By default we process during the After Initialise.
		// If the option is activate, it means that it has been already processed so we do not need to do it twice !
		//
		if($this->params->get('after_init', 1))
			return;

		$this->process();
	}

	/**
	 * Processing function
	 */
	protected function process() {

		// Retrive the called URL
		$path = $this->currentURL();

		// Check if the URL is for the API
		$apiStart = '?option=com_hikashop&ctrl=shipstation';

		// keyword find or not in the Url? if not process end here
		if(strpos($path, $apiStart) === false)
			return;

		// ==========================================
		// Application Test
		// 
		// http://localhost/Joomla%203.3.3/index.php?option=com_hikashop&ctrl=shipstation&SS-UserName=philippe&SS-Password=domoshima&action=export&test=1&start_date=10%2f8%2f2015+12%3a47&end_date=11%2f26%2f2015+18%3a55&page=1
		//
		// ==========================================

		// Load HikaShop
		//
		$hikashopHelper = rtrim(JPATH_ADMINISTRATOR, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_hikashop' . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'helper.php';
		if(!file_exists($hikashopHelper) || !include_once($hikashopHelper))
			exit;

		// Get data from shipstation XML
		//
		$user = $this->params->get('user', '');
		$password = $this->params->get('pass', '');
		$debug = (int)$this->params->get('debug', 0);

		// Extract username and password from the URL
		preg_match('#SS-UserName=([^&]+)#i', $path, $matches);
		$shipUser = $matches[1];
		preg_match('#SS-Password=([^&]+)#i', $path, $matches);
		$shipPass = $matches[1];

		// Check that User and Password match
		//
		if($shipUser != $user || $shipPass != $password) {
			if($debug) {
				echo 'Unexpected username and password';

				$this->writeToLog(
					'Shipstation notification with unexpected username and password'."\r\n".
					'Shipstation Username: '.$shipUser."\r\n".
					'Shipstation Password: '.$shipPass."\r\n".
					$path
				);
			}

			// Quit Shipstation plugin
			exit;
		}

		// Manage the requested action
		//
		preg_match('#action=([^&]+)#i', $path, $matches);
		$shipstatus = $matches[1];

		switch($shipstatus) {
			// Going in export mode
			case 'export':
				$this->processExport($path);
				break;

			// Ship DB need an data unpdate
			case 'shipnotify':
				$this->processShipNotify($path);
				break;

			// Invalid or unknown action.
			default:
				if($debug)
					$this->writeToLog('Unknow action/status from shipstation: ' . $shipstatus);
				break;
		}

		// We don't want that Joomla or another plugin can display some content
		exit;
	}

	/**
	 *
	 */
	private function processExport($path) {
		$debug = (int)$this->params->get('debug', 0);
		$db = JFactory::getDBO();

		// Extract start and end date/hour from url :
		preg_match('#start_date=([^&]+)#i', $path, $start);
		preg_match('#end_date=([^&]+)#i', $path, $end);

		// Decode date/hour and stand to unix :
		$start_unix = strtotime( urldecode($start[1]) );
		$end_unix = strtotime( urldecode($end[1]) );

		// Invalid dates : no processing.
		//
		if($start_unix === false || $start_unix === -1 || $end_unix === false || $end_unix === -1) {
			if($debug)
				$this->writeToLog('Ship Station error: impossible to get start and end dates.');

			exit;
		}

		//ORDER SQL REQUEST & TREATMENTS :
		//SQL Query in order data base :
		//Caution : for some test the SQL is : ..." WHERE `order_created`
		//AND for products mode : the SQL is : ..." WHERE `order_invoice_created`
		//$SqlOrder = "order_created" OR "order_invoice_created"

		$sql_order = $this->params->get('sqlorder', '');
		if(empty($sql_order))
			$sql_order = 'order_invoice_created';

		if(!HIKASHOP_J30)
			$sql_order = $db->nameQuote($sql_order);
		else
			$sql_order = $db->quoteName($sql_order);

		$query = 'SELECT * FROM ' . hikashop_table('order') .
			' WHERE (' . $sql_order . ' > ' . (int) $start_unix . ') AND (' . $sql_order . ' < ' . (int)$end_unix . ') AND (`order_shipping_address_id` != 0)';
		$db->setQuery($query);
		$orders = $db->loadObjectList('order_id');

		//
		//
		if(empty($orders)) {
			if($debug) {
				$this->writeToLog(
					'no orders database matched with this dates'."\r\n".
					'Start Date: '.$start[1].' // End date: '.$end[1]
				);
			}

			echo '<'.'?xml version="1.0" encoding="utf-8"?'.'><Orders></Orders>';
			exit;
		}

		//
		//
		//
		$order_id = array_keys($orders);
		JArrayHelper::toInteger($order_id);

		$address_id = array();
//		$user_id = array();

		foreach($orders as $k => $v) {
			//
			$i = (int)$v->order_shipping_address_id;
			if($i > 0)
				$address_id[$i] = $i;

			//
//			$i = (int)$value->order_user_id;
//			if($i > 0)
//				$user_id[$i] = $i;

			//
			$orders[$k]->products = array();
		}

		//
		// Loading Order Products
		//
		$query = 'SELECT * FROM ' . hikashop_table('order_product') . ' WHERE `order_id` IN (' . implode(',', $order_id ) . ')';
		$db->setQuery($query);
		$order_products = $db->loadObjectList();

		if(empty($order_products)) {
			if($debug) {
				$this->writeToLog('Ship Station error: impossible to read order_product database');
			}

			echo '<'.'?xml version="1.0" encoding="utf-8"?'.'><Orders></Orders>';
			exit;
		}

		//
		// Loading Addresses
		//
		$query = 'SELECT * FROM ' . hikashop_table('address') . ' WHERE `address_id` IN (' . implode(',', $address_id) . ')';
		$db->setQuery($query);
		$addresses = $db->loadObjectList('address_id');

		if(empty($addresses)) {
			if($debug) {
				$this->writeToLog('Ship Station error: impossible to read addresses database');
			}

			echo '<'.'?xml version="1.0" encoding="utf-8"?'.'><Orders></Orders>';
			exit;
		}

		//
		// Loading Users
		// TODO : According to what is made later, there is no need to load the users in the database, we just need the user_id ; element that we already have in the order.
		//
//		$query = 'SELECT * FROM ' . hikashop_table('user') . ' WHERE `user_id` IN (' . implode(',', $user_id ) . ')';
//		$db->setQuery($query);
//		$users = $db->loadObjectList('user_id');
//
//		if(empty($users)) {
//			if($debug) {
//				$this->writeToLog("Ship Station error: impossible to read user database");
//			}
//
//			echo '<'.'?xml version="1.0" encoding="utf-8"?'.'><Orders></Orders>';
//			exit;
//		}

		// Add order_product data arrays data in orders
		//
		foreach($order_products as $k => $v) {
			$orders[ (int)$v->order_id]->products[] =& $order_products[$k];
		}

		// Initialize the XML data which will be send to Shipstation
		//
		$xml_data = '<'.'?xml version="1.0" encoding="utf-8"?'.'>
<Orders>';

		// Process all orders to generate the appropriate XML
		//
		foreach($orders as $order) {

			// Skip order without products (items), invalid address or invalid user.
			//
			if(!isset($order->products) || !isset($addresses[$order->order_shipping_address_id])  ) {
				continue;
			}

			// Get address for each order
			$address =& $addresses[$order->order_shipping_address_id];

			//Get zone codeS for each order:
			$countryCode =  $this->get_zone_code_2 ($address->address_country);

			//Get user for each order :
			// $user =& $users[$order->order_user_id];

			// -- NOTICE --
			// The order number will be use by shipstation as order identifier.
			// Later Hikashop will need the order id to find the order BUT for shipnotify will only return the <OrderNumber>
			// That's why we switch these two data.

			// Generate the Order XML data
			//
			$xml_data .= '
	<Order>
		<OrderID><![CDATA[' . $order->order_number . ']]></OrderID>
		<OrderNumber><![CDATA[' . (int)$order->order_id . ']]></OrderNumber>
		<OrderDate>' . date('M/d/Y H:i:s', (int)$order->order_created) . '</OrderDate>
		<OrderStatus><![CDATA[' . $order->order_status . ']]></OrderStatus>
		<LastModified>' . date('M/d/Y H:i:s', (int)$order->order_modified) . '</LastModified>
		<OrderTotal>' . round($order->order_full_price, 2) . '</OrderTotal>
		<ShippingAmount>' . round($order->order_shipping_price, 2) . '</ShippingAmount>
		<Customer>
			<CustomerCode><![CDATA[' . (int)$order->order_user_id . ']]></CustomerCode>
			<BillTo>
				<Name><![CDATA[' . $address->address_lastname . ']]></Name>
			</BillTo>
			<ShipTo>
				<Name><![CDATA[' . $address->address_lastname . ']]></Name>
				<Address1><![CDATA[' . $address->address_street . ']]></Address1>
				<Address2><![CDATA[' . $address->address_street2 . ']]></Address2>
				<City><![CDATA[' . $address->address_city . ']]></City>
				<State><![CDATA[' . $address->address_state . ']]></State>
				<PostalCode><![CDATA[' . $address->address_post_code . ']]></PostalCode>
				<Country><![CDATA[' . $countryCode . ']]></Country>
			</ShipTo>
		</Customer>
		<Items>';

			// List all items of the current order
			//
			foreach($order->products as $product) {
				$xml_data .= '
			<Item>
				<SKU><![CDATA[' . $product->order_product_code . ']]></SKU>
				<Name><![CDATA[' . $product->order_product_name . ']]></Name>
				<Quantity>' . (int)$product->order_product_quantity . '</Quantity>
				<UnitPrice>' . round( ($product->order_product_price + $product->order_product_tax), 2) . '</UnitPrice>
			</Item>';
			}

			// Close the current order in the XML
			//
			$xml_data .= '
		</Items>
	</Order>';

		}
		unset($address);
		unset($countryCode);

		// Finalize the XML content
		$xml_data .= '
</Orders>';

		// Store the XML data into the log file
		if($debug) {
			$this->writeToLog(
				'Sent XML to shipstation'."\r\n".
				htmlentities($xml_data)
			);
		}

		// Send to shipstation XML response:
		echo $xml_data;
	}

	/**
	 *
	 */
	private function processShipNotify($path) {
		$debug = (int)$this->params->get('debug', 0);

		/*
		 * 
		https://localhost/index.php?option=com_hikashop&ctrl=shipstation&SS-UserName=philippe&SS-Password=domoshima&action=shipnotify&order_number=157&carrier=USPS&service=USPS+Priority+Mail&tracking_number=9511343223432432432
		 */

		// Read the "order_number" in URL ; which will be the order_id in HikaShop
		//
		preg_match('#order_number=([^&]+)#i', $path, $matches);
		$order_id = (int)$matches[1];

		//Get the XML :
		$postdata = file_get_contents('php://input');

		// Simulate shipnotify (data import from shipstation)
/*
		$postdata = '<'.'?xml version="1.0" encoding="utf-8"?'.'>
<ShipNotice>
	<OrderNumber>7</OrderNumber>
	<OrderID>G1W57</OrderID>
	<CustomerCode>customer@mystore.com</CustomerCode>
	<LabelCreateDate>12/8/2011 12:56 </LabelCreateDate>
	<ShipDate>12/8/2011</ShipDate>
	<Carrier>USPS</Carrier>
	<Service>Priority Mail</Service>
	<TrackingNumber>1Z909084330298430820</TrackingNumber>
	<ShippingCost>4.95</ShippingCost>
	<Recipient>
		<Name>The President</Name>
		<Company>US Govt</Company>
		<Address1>1600 Pennsylvania Ave</Address1>
		<Address2></Address2>
		<City>Washington</City>
		<State>DC</State>
		<PostalCode>20500</PostalCode>
		<Country>US</Country>
	</Recipient>
	<Items>
		<Item>
			<SKU>FD88821</SKU>
			<Name>My Product Name</Name>
			<Quantity>2</Quantity>
		</Item>
	</Items>
</ShipNotice>
';
		$postdata = str_replace(array("\r\n","\r","\n","\t"), '', $postdata);
		//
*/
		// Read the "orderID" in Shipstation data ; which will be the order_number in HikaShop
		//
		preg_match('#<orderID>([^<]+)#i', $postdata, $matches);
		$order_number = $matches[1];

		// Get the order with the order_number from shipstation (order_id for hikashop)
		//
		$orderClass = hikashop_get('class.order');
		$order = $orderClass->get( (int)$order_id );

		if(empty($order)) {
			if($debug) {
				$this->writeToLog(
					'No Order found link to order id from shipstation'."\r\n".
					'view on order id shipstation: '. (int)$order_id
				);
			}

			//
			// TODO : Are you sure that it should be displayed as content and not as header ?
			//
			echo 'HTTP/1.0 200 OK';
			exit;
		}

		//
		//
		preg_match('#<TrackingNumber>([^<]+)#i', $postdata, $matches);
		$TrackingNumber = trim($matches[1]);

		preg_match('#<Carrier>([^<]+)#i', $postdata, $matches);
		$Carrier = trim($matches[1]);

		preg_match('#<Service>([^<]+)#i', $postdata, $matches);
		$Service = trim($matches[1]);

		// Check if order number & customer name match with hikashop
		//
		if($order_number != $order->order_number) {
			$this->writeToLog(
				'Order number mismatch!'."\r\n".
				'from hikashop DB: '.$order->order_number."\r\n".
				'from shipstation: '.$order_number
			);
		}

		// Create data string from shipstation data
		//
		// TODO : Could be great to have some kind of TAG as prefix so it will be easy to find it later. Like: [SHIPSTATION]
		//
		$details = $Carrier . '::' . $Service . '::' . $TrackingNumber;

		// Update status order with shipstation data :
		$update = new stdClass();
		$update->order_id = $order->order_id;
		$update->history = new stdClass();

		$order_status = trim($this->params->get('orderstatus', ''));
		$notify_customer = (int)$this->params->get('customermail', 0);
		if(!empty($order_status)) {
			$update->order_status = $order_status;
			$update->history->history_notified = $notify_customer;
		}

		// Update order data
		$update->history->history_data = $details;
		$orderClass->save($update);

		// Write result and details in log file :
		if($debug)
			$this->writeToLog('order No. : ' . $order->order_id . "\r\n" . $details . "\r\n");

		// Successfully update process, tell it to shipstation :
		//
		// TODO : Are you sure that it should be displayed as content and not as header ?
		//
		echo 'HTTP/1.0 200 OK';
	}

	/**
	 * sent data to customer email :
	 */
	public function onAfterOrderProductsListingDisplay(&$order) {
		$app = JFactory::getApplication();
		if(!$app->isAdmin())
			return;

		$option = JRequest::getString('option', '');
		$ctrl = JRequest::getCmd('ctrl', '');
		$task = JRequest::getString('task', '');

		// TODO : It should be better to list where we want to display the content instead of excluding elements.
		// Things can evolve and other components can also call this trigger (like HikaMarket).
		//
		if($option != 'com_hikashop' || ($ctrl == 'order' && in_array($task, array('show', 'invoice'))))
			return;

		$db = JFactory::getDBO();

		// Load the history data
		//
		// TODO : Because the order status is an option in the plugin, it should be the same here or there should be a link.
		// Loading only the first element might not be the perfect solution, specially if the admin change the order in the backend.
		//
		$query = 'SELECT history_data FROM ' . hikashop_table('history') .
			' WHERE history_new_status = '.$db->Quote('shipped').' AND history_data != '.$db->Quote('').' AND history_order_id = ' .(int)$order->order_id.
			' ORDER BY history_created ASC';
		$db->setQuery($query);
		$history = $db->loadResult();

		if(empty($history))
			return;

		//
		list($Carrier, $Service, $TrackingNumber) = explode('::', $history, 3);
		echo JText::_('CARRIER').': '.$Carrier.' '.JText::_('SERVICE').': '.$Service.' '.JText::_('TRACKING_NUMBER').': '.$TrackingNumber.'<br />';
	}

	/**
	 * For dynamic translation of email and back end orders histories data
	 */
	public function onHistoryDisplay(&$histories) {

		// Display only for email, if there is some data to display
		//
		$option = JRequest::getString('option', '');
		$ctrl = JRequest::getCmd('ctrl', '');
		$task = JRequest::getString('task', '');
		if(empty($histories) || $option != 'com_hikashop' || $ctrl != 'order' || $task != 'show')
			return;

		foreach($histories as $k => $history) {
			if(empty($history->history_data) || strpos($history->history_data, '::') === false)
				continue;

			// Construct string with shipstation data:
			list($Carrier, $Service, $TrackingNumber) = explode('::', $history->history_data, 3);
			$histories[$k]->history_data = JText::_('CARRIER').': '.$Carrier.'<br/>' .
				JText::_('SERVICE').': '.$Service.'<br/>' .
				JText::_('TRACKING_NUMBER').': '.$TrackingNumber.'<br />';
		}
	}

	/**
	 * Get the current url
	 * From HikaShop helper.php
	 */
	private function currentURL($safe = true) {
		if(!empty($_SERVER["REDIRECT_URL"]) && preg_match('#.*index\.php$#',$_SERVER["REDIRECT_URL"])
		  && empty($_SERVER['QUERY_STRING']) && (empty($_SERVER['REDIRECT_QUERY_STRING']) ||
		  strpos($_SERVER['REDIRECT_QUERY_STRING'],'&') === false) && !empty($_SERVER["REQUEST_URI"])) {

			$requestUri = $_SERVER["REQUEST_URI"];
			if (!empty($_SERVER['REDIRECT_QUERY_STRING'])) $requestUri = rtrim($requestUri,'/').'?'.$_SERVER['REDIRECT_QUERY_STRING'];
		}
		elseif(!empty($_SERVER["REDIRECT_URL"]) && (isset($_SERVER['QUERY_STRING']) || isset($_SERVER['REDIRECT_QUERY_STRING']))) {
			$requestUri = $_SERVER["REDIRECT_URL"];
			if(!empty($_SERVER['REDIRECT_QUERY_STRING']))
				$requestUri = rtrim($requestUri,'/').'?'.$_SERVER['REDIRECT_QUERY_STRING'];
			elseif (!empty($_SERVER['QUERY_STRING']))
				$requestUri = rtrim($requestUri,'/').'?'.$_SERVER['QUERY_STRING'];
		}
		elseif(isset($_SERVER["REQUEST_URI"])) {
			$requestUri = $_SERVER["REQUEST_URI"];
		}
		else {
			$requestUri = $_SERVER['PHP_SELF'];
			if(!empty($_SERVER['QUERY_STRING']))
				$requestUri = rtrim($requestUri,'/').'?'.$_SERVER['QUERY_STRING'];
		}

		$result = 'http://'.$_SERVER["HTTP_HOST"].$requestUri;
		if($safe)
			$result = str_replace(array('"',"'",'<','>',';'),array('%22','%27','%3C','%3E','%3B'),$result);
		return $result;
	}

	/**
	 * Add error data in the log files (debug)
	 * From HikaShop helper.php
	 */
	private function writeToLog($data = null) {
		$dbg = ($data === null) ? ob_get_clean() : $data;
		if(!empty($dbg)) {
			$dbg = '-- ' . date('m.d.y H:i:s') . ' --'. (empty($this->name) ? ('['.$this->name.']') : '') . "\r\n" . $dbg;

			jimport('joomla.filesystem.file');
			$config = hikashop_config();
			$file = $config->get('payment_log_file', '');
			$file = rtrim(JPath::clean(html_entity_decode($file)), DS . ' ');
			if(!preg_match('#^([A-Z]:)?/.*#',$file) && (!$file[0] == '/' || !file_exists($file)))
				$file = JPath::clean(HIKASHOP_ROOT . DS . trim($file, DS . ' '));
			if(!empty($file) && defined('FILE_APPEND')) {
				if(!file_exists(dirname($file))) {
					jimport('joomla.filesystem.folder');
					JFolder::create(dirname($file));
				}
				file_put_contents($file, $dbg, FILE_APPEND);
			}
		}
		if($data === null)
			ob_start();
	}

	function get_zone_code_2 ($address_country) {

		//Get country zone code:
		$class =hikashop_get('class.zone');
		$zone = $class->get($address_country);
		$zone = $zone->zone_code_2;

		return ($zone);
	}
}